﻿using Styx;
using Styx.Common;
using Styx.WoWInternals;
using Styx.WoWInternals.WoWObjects;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Numerics;

#region Redundancy
using L = DemoLock.Helpers.Log;
using S = DemoLock.Core.Spell;
using SB = DemoLock.Helpers.SpellBook;
#endregion

namespace DemoLock.Core
{
    class Unit
    {
        private static LocalPlayer Me { get { return StyxWoW.Me; } }
        private static WoWUnit currentTarget { get { return StyxWoW.Me.CurrentTarget; } }

        
        //Active Enemies
        public static IEnumerable<WoWUnit> activeEnemies(Vector3 fromLocation, double Range)
        {
            var Hostile = enemyCount;
            return Hostile != null ? Hostile.Where(x => x.Location.DistanceSquared(fromLocation) <= Range * Range) : null;
        }

        private static List<WoWUnit> enemyCount { get; set; }
        
        public static void enemyAnnex(double Range)
        {
            enemyCount.Clear();
            foreach (var u in surroundingEnemies())
            {
                if (u == null || !u.IsValid)//<~ Use isUnitValid(u, Range) instead of the frist five of these first calls?
                    continue;
                if (!u.IsAlive || u.DistanceSqr > Range * Range)
                    continue;
                if (!u.Attackable || !u.CanSelect)
                    continue;
                if (u.IsFriendly)
                    continue;
                if (u.IsNonCombatPet && u.IsCritter)
                    continue;
                enemyCount.Add(u);
            }
        }

        private static IEnumerable<WoWUnit> surroundingEnemies() { return ObjectManager.GetObjectsOfTypeFast<WoWUnit>(); }

        static Unit() { enemyCount = new List<WoWUnit>();    }


        //Aura Detection
        public static bool auraExists(WoWUnit Unit, int auraID, bool isMyAura = false)
        {
            try
            {
                if (Unit == null || !Unit.IsValid)
                    return false;
                WoWAura Aura = isMyAura ? Unit.GetAllAuras().FirstOrDefault(A => A.SpellId == auraID && A.CreatorGuid == Me.Guid) : Unit.GetAllAuras().FirstOrDefault(A => A.SpellId == auraID);
                return Aura != null;
            }
            catch (Exception xException)
            {
                L.diagnosticsLog("Exception in auraExists(); ", xException);
                return false;
            }
        }

        public static uint auraStacks(WoWUnit Unit, int auraID, bool isMyAura = false)
        {
            try
            {
                if (Unit == null || !Unit.IsValid)
                    return 0;
                WoWAura Aura = isMyAura ? Unit.GetAllAuras().FirstOrDefault(A => A.SpellId == auraID && A.CreatorGuid == Me.Guid) : Unit.GetAllAuras().FirstOrDefault(A => A.SpellId == auraID);
                return Aura != null ? Aura.StackCount : 0;
            }
            catch (Exception xException)
            {
                L.diagnosticsLog("Exception in auraStacks(); ", xException);
                return 0;
            }
        }

        public static double auraTimeLeft(WoWUnit Unit, int auraID, bool isMyAura = false)
        {
            try
            {
                if (Unit == null || !Unit.IsValid)
                    return 0;
                WoWAura Aura = isMyAura ? Unit.GetAllAuras().FirstOrDefault(A => A.SpellId == auraID && A.CreatorGuid == Me.Guid) : Unit.GetAllAuras().FirstOrDefault(A => A.SpellId == auraID);
                return Aura != null ? Aura.TimeLeft.TotalMilliseconds : 0;
            }
            catch (Exception xException)
            {
                L.diagnosticsLog("Exception in auraExists(); ", xException);
                return 9999;
            }
        }

        public static IEnumerable<WoWUnit> wildImps() { return Me.Minions.Where(min => min.Name == "Wild Imp"); }
        public static IEnumerable<WoWUnit> wildImpsWithDE() { return Me.Minions.Where(min => min.Name == "Wild Imp" && min.HasAura(SB.demonicEmpowerment)); }
        public static IEnumerable<WoWUnit> dreadStalkersWithDE() { return Me.Minions.Where(min => min.Name == "Dreadstalker" && min.HasAura(SB.demonicEmpowerment)); }


        #region Cache

        public static double doomTimeLeft;
        public static bool useArtifact;

        public static int wildImpCount;
        public static int wildImpWithDemEmpCount;
        public static int dreadStalkersWithDemEmpCount;

        public static void Cache()
        {
            doomTimeLeft = auraTimeLeft(currentTarget, SB.doom);

            wildImpCount = wildImps().Count();
            wildImpWithDemEmpCount = wildImpsWithDE().Count();
            dreadStalkersWithDemEmpCount = dreadStalkersWithDE().Count();

            useArtifact = wildImpWithDemEmpCount >= 5 && dreadStalkersWithDemEmpCount >= 2;

            
        }
        #endregion

        //Unit Eval
        public static bool isUnitValid(WoWUnit Unit, double Range) { return Unit != null && Unit.IsValid && Unit.IsAlive && Unit.Attackable && Unit.DistanceSqr <= Range * Range; } //<~ Can we skim this down a bit or remove it completely?
        public static bool isUnitValid(WoWUnit Unit) { return Unit != null && Unit.IsValid && Unit.IsAlive && Unit.Attackable; } //<~ Can we skim this down a bit or remove it completely?
        public static double luaGetPower()
        {
            try
            {
                using (StyxWoW.Memory.AcquireFrame()) { return Lua.GetReturnVal<int>("return UnitPower(\"player\");", 0); }
            }
            catch (Exception xException)
            {
                L.diagnosticsLog("Exception in playerPower(); ", xException);
                return Me.CurrentPower;
            }
        }
    }
}
